@@ -1,5 +1,5 @@
 /* Data structures and API for event locations in GDB.
-   Copyright (C) 2013-2017 Free Software Foundation, Inc.
+   Copyright (C) 2013-2018 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -66,6 +66,17 @@ enum event_location_type
   PROBE_LOCATION
 };
 
+/* A traditional linespec.  */
+
+struct linespec_location
+{
+  /* Whether the function name is fully-qualified or not.  */
+  symbol_name_match_type match_type;
+
+  /* The linespec.  */
+  char *spec_string;
+};
+
 /* An explicit location.  This structure is used to bypass the
    parsing done on linespecs.  It still has the same requirements
    as linespecs, though.  For example, source_filename requires
@@ -79,6 +90,9 @@ struct explicit_location
   /* The function name.  Malloc'd.  */
   char *function_name;
 
+  /* Whether the function name is fully-qualified or not.  */
+  symbol_name_match_type func_name_match_type;
+
   /* The name of a label.  Malloc'd.  */
   char *label_name;
 
@@ -107,7 +121,7 @@ extern char *
 
 /* Return a string representation of the LOCATION.
    This function may return NULL for unspecified linespecs,
-   e.g, LOCATION_LINESPEC and addr_string is NULL.
+   e.g, LINESPEC_LOCATION and spec_string is NULL.
 
    The result is cached in LOCATION.  */
 
@@ -127,12 +141,13 @@ typedef std::unique_ptr<event_location, event_location_deleter>
 
 /* Create a new linespec location.  */
 
-extern event_location_up new_linespec_location (char **linespec);
+extern event_location_up new_linespec_location
+  (const char **linespec, symbol_name_match_type match_type);
 
-/* Return the linespec location (a string) of the given event_location
-   (which must be of type LINESPEC_LOCATION).  */
+/* Return the linespec location of the given event_location (which
+   must be of type LINESPEC_LOCATION).  */
 
-extern const char *
+extern const linespec_location *
   get_linespec_location (const struct event_location *location);
 
 /* Create a new address location.
@@ -205,33 +220,60 @@ extern event_location_up
    This function is intended to be used by CLI commands and will parse
    explicit locations in a CLI-centric way.  Other interfaces should use
    string_to_event_location_basic if they want to maintain support for
-   legacy specifications of probe, address, and linespec locations.  */
+   legacy specifications of probe, address, and linespec locations.
+
+   MATCH_TYPE should be either WILD or FULL.  If -q/--qualified is specified
+   in the input string, it will take precedence over this parameter.  */
+
+extern event_location_up string_to_event_location
+  (const char **argp, const struct language_defn *langauge,
+   symbol_name_match_type match_type = symbol_name_match_type::WILD);
+
+/* Like string_to_event_location, but does not attempt to parse
+   explicit locations.  MATCH_TYPE indicates how function names should
+   be matched.  */
 
 extern event_location_up
-  string_to_event_location (char **argp,
-			    const struct language_defn *langauge);
+  string_to_event_location_basic (const char **argp,
+				  const struct language_defn *language,
+				  symbol_name_match_type match_type);
 
-/* Like string_to_event_location, but does not attempt to parse explicit
-   locations.  */
+/* Structure filled in by string_to_explicit_location to aid the
+   completer.  */
+struct explicit_completion_info
+{
+  /* Pointer to the last option found.  E.g., in "b -sou src.c -fun
+     func", LAST_OPTION is left pointing at "-fun func".  */
+  const char *last_option = NULL;
 
-extern event_location_up
-  string_to_event_location_basic (char **argp,
-				  const struct language_defn *language);
+  /* These point to the start and end of a quoted argument, iff the
+     last argument was quoted.  If parsing finds an incomplete quoted
+     string (e.g., "break -function 'fun"), then QUOTED_ARG_START is
+     set to point to the opening \', and QUOTED_ARG_END is left NULL.
+     If the last option is not quoted, then both are set to NULL. */
+  const char *quoted_arg_start = NULL;
+  const char *quoted_arg_end = NULL;
+
+  /* True if we saw an explicit location option, as opposed to only
+     flags that affect both explicit locations and linespecs, like
+     "-qualified".  */
+  bool saw_explicit_location_option = false;
+};
 
 /* Attempt to convert the input string in *ARGP into an explicit location.
    ARGP is advanced past any processed input.  Returns an event_location
    (malloc'd) if an explicit location was successfully found in *ARGP,
    NULL otherwise.
 
-   IF !DONT_THROW, this function may call error() if *ARGP looks like
-   properly formed input, e.g., if it is called with missing argument
-   parameters or invalid options.  If DONT_THROW is non-zero, this function
-   will not throw any exceptions.  */
+   If COMPLETION_INFO is NULL, this function may call error() if *ARGP
+   looks like improperly formed input, e.g., if it is called with
+   missing argument parameters or invalid options.  If COMPLETION_INFO
+   is not NULL, this function will not throw any exceptions.  */
 
 extern event_location_up
   string_to_explicit_location (const char **argp,
-			       const struct language_defn *langauge,
-			       int dont_throw);
+			       const struct language_defn *language,
+			       explicit_completion_info *completion_info);
 
 /* A convenience function for testing for unset locations.  */
 
